-- Greg Stitt
-- University of Florida
-- EEL 5934/4930 Reconfigurable Computing
--
-- File: datapath.vhd
--
-- Description: This file implements the datapath for the Fibonacci
-- calculator.
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity DATAPATH is
  generic (width :     natural := 32);
  port(clk, rst  : in  std_logic;
       n         : in  unsigned(width-1 downto 0);
       result    : out unsigned(width-1 downto 0);

       -- control signals
       i_sel, x_sel, y_sel               : in  std_logic;
       i_ld, x_ld, y_ld, n_ld, result_ld : in  std_logic;
       i_le_n                            : out std_logic );
end DATAPATH;

architecture str of DATAPATH is

  component REG
    generic (width    :     natural := 32);
    port(clk, rst, en : in  std_logic;
         input        : in  unsigned(width-1 downto 0);
         output       : out unsigned(width-1 downto 0));
  end component;

  component MUX_2_1
    generic (width       :     natural := 32);
    port( input1, input2 : in  unsigned(width-1 downto 0);
          sel            : in  std_logic;
          output         : out unsigned(width-1 downto 0));
  end component;

  component LE
    generic (width       :     natural := 32);
    port( input1, input2 : in  unsigned(width-1 downto 0);
          le             : out std_logic );
  end component;

  component ADD
    generic (width       :     natural := 32);
    port( input1, input2 : in  unsigned(width-1 downto 0);
          output         : out unsigned(width-1 downto 0) );
  end component;

  signal mux_i_out, mux_x_out, mux_y_out            : unsigned(width-1 downto 0);
  signal reg_i_out, reg_x_out, reg_y_out, reg_n_out : unsigned(width-1 downto 0);
  signal add1_out, add2_out                         : unsigned(width-1 downto 0);

  constant C1 : unsigned(width-1 downto 0) := conv_unsigned(1, width);
  constant C3 : unsigned(width-1 downto 0) := conv_unsigned(3, width);

begin

  -- instantiate the muxes
  MUX_I : MUX_2_1 generic map (width) port map (C3, add1_out, i_sel, mux_i_out);
  MUX_X : MUX_2_1 generic map (width) port map (C1, reg_y_out, x_sel, mux_x_out);
  MUX_Y : MUX_2_1 generic map (width) port map (C1, add2_out, y_sel, mux_y_out);

  --instantiate the registers
  REG_I : REG generic map (width) port map (clk, rst, i_ld, mux_i_out, reg_i_out);
  REG_X : REG generic map (width) port map (clk, rst, x_ld, mux_x_out, reg_x_out);
  REG_Y : REG generic map (width) port map (clk, rst, y_ld, mux_y_out, reg_y_out);
  REG_N : REG generic map (width) port map (clk, rst, n_ld, n, reg_n_out);
  REG_R : REG generic map (width) port map (clk, rst, result_ld, reg_y_out, result);

  -- instantiate the adders
  ADD1 : ADD generic map (width) port map (reg_i_out, C1, add1_out);
  ADD2 : ADD generic map (width) port map (reg_x_out, reg_y_out, add2_out);

  -- instantiate the less than/equal to comparator
  LE1 : LE generic map (width) port map (reg_i_out, reg_n_out, i_le_n);

end str;
